@@ -1,5 +1,4 @@
 # coding: utf-8
-from __future__ import unicode_literals, division, absolute_import, print_function
 
 
 class PathError(Exception):
@@ -63,11 +62,6 @@ class OCSPFetchError(OCSPValidationError):
     pass
 
 
-class SoftFailError(Exception):
-
-    pass
-
-
 class ValidationError(Exception):
 
     pass
